/* 
 * Project     : WinAmp USB LightCtrl via USB
 * Author      : frydaysoft.de (based on an example from Stanimir Bonev.)
 * File name   : nrf2401l.h
 * Description : nRF24L01 registers definitions.
 *  
 * Copyright: (c) 2013 frydaysoft.de 
 * License  : Proprietary, free under certain conditions. See Documentation.
 *
 * Creation Date: 2013-04-27
 */

#ifndef NRF24L01_H
#define NRF24L01_H

//********************************************************************************************************************//
// SPI(nRF24L01) commands
#define NRF_CMD_READ_REG            0x00  // 00..1F Define read command to register
#define NRF_CMD_WRITE_REG           0x20  // 20..3F Define write command to register
#define NRF_CMD_RD_RX_PLOAD         0x61  // Define RX payload register address
#define NRF_CMD_WR_TX_PLOAD         0xA0  // Define TX payload register address
#define NRF_CMD_FLUSH_TX            0xE1  // Define flush TX register command
#define NRF_CMD_FLUSH_RX            0xE2  // Define flush RX register command
#define NRF_CMD_REUSE_TX_PL         0xE3  // Define reuse TX payload register command
#define NRF_CMD_ACTIVE              0x50  // used with: R_RX_PL_WID, W_ACK_PAYLOAD, W_TX_PAYLOAD_NOACK
#define NRF_CMD_ACTIVE_DATA         0x73  // sequence token for NRF_CMD_ACTIVE
#define NRF_CMD_RD_RX_PLOAD_WID     0x60  // Read RX-payload width for the top R_RX_PAYLOAD in the RX FIFO.
#define NRF_CMD_WR_ACK_PAYLOAD      0xA8  // A8..AC: Used in RX mode.
#define NRF_CMD_WR_TX_PAYLOAD_NACK  0xB0  // Used in TX mode. Disables AUTOACK on this specific packet.
#define NRF_CMD_NOP                 0xFF  // Define No Operation, might be used to read status register

//********************************************************************************************************************//
// SPI(nRF24L01) registers(addresses)
#define NRF_REG_CONFIG          0x00  // 'Config' register address
        #define NRF_SFT_CONFIG__PRIM_RX         (0)
        #define NRF_MSK_CONFIG__PRIM_RX         (1<<NRF_SFT_CONFIG__PRIM_RX)
        #define NRF_SFT_CONFIG__PWR_UP          (1)
        #define NRF_MSK_CONFIG__PWR_UP          (1<<NRF_SFT_CONFIG__PWR_UP)
        #define NRF_SFT_CONFIG__CRCO            (2)
        #define NRF_MSK_CONFIG__CRCO            (1<<NRF_SFT_CONFIG__CRCO)
        #define NRF_SFT_CONFIG__EN_CRC          (3)
        #define NRF_MSK_CONFIG__EN_CRC          (1<<NRF_SFT_CONFIG__EN_CRC)
#define NRF_REG_EN_AA           0x01  // 'Enable Auto Acknowledgment' register address
#define NRF_REG_EN_RXADDR       0x02  // 'Enabled RX addresses' register address
#define NRF_REG_SETUP_AW        0x03  // 'Setup address width' register address
#define NRF_REG_SETUP_RETR      0x04  // 'Setup Auto. Retrans' register address
        #define NRF_SFT_SETUP_RETR__ARD         (4)
        #define NRF_MSK_SETUP_RETR__ARD         (0x0F<<NRF_SFT_SETUP_RETR__ARD)
        #define NRF_SFT_SETUP_RETR__ARC         (0)
        #define NRF_MSK_SETUP_RETR__ARC         (0x0F<<NRF_SFT_SETUP_RETR__ARC)
#define NRF_REG_RF_CH           0x05  // 'RF channel' register address
        #define NRF_SFT_RF_CH__RANGE            (0)
        #define NRF_MSK_RF_CH__RANGE            (0x7F<<NRF_SFT_RF_CH__RANGE)
#define NRF_REG_RF_SETUP        0x06  // 'RF setup' register address
        #define NRF_SFT_RF_SETUP__RF_LNA        (0)
        #define NRF_MSK_RF_SETUP__RF_LNA        (1<<NRF_SFT_RF_SETUP__RF_LNA)
        #define NRF_SFT_RF_SETUP__RF_DR         (3)
        #define NRF_MSK_RF_SETUP__RF_DR         (1<<NRF_SFT_RF_SETUP__RF_DR)
#define NRF_REG_STATUS          0x07  // 'Status' register address
        #define NRF_SFT_STATUS__TX_FULL         (0)
        #define NRF_MSK_STATUS__TX_FULL         (1<<NRF_SFT_STATUS__TX_FULL)
        #define NRF_SFT_STATUS__RX_P_NO         (1)
        #define NRF_MSK_STATUS__RX_P_NO         (7<<NRF_SFT_STATUS__RX_P_NO)
        #define NRF_GET_RX_NO(val)    ((val & NRF_MSK_STATUS__RX_P_NO)>>NRF_SFT_STATUS__RX_P_NO)
        #define NRF_MAX_RT_IRQ        (1<<4)
        #define NRF_TX_DS_IRQ         (1<<5)
        #define NRF_RX_DR_IRQ         (1<<6)
        #define NRF_ALL_IRQ           ((NRF_MAX_RT_IRQ) | (NRF_TX_DS_IRQ) | (NRF_RX_DR_IRQ))
        #define NRF_MASK_ALL_IRQ(val) (val & NRF_ALL_IRQ)
#define NRF_REG_OBSERVE_TX      0x08  // 'Observe TX' register address
        #define NRF_SFT_OBSERVE_TX__PLOS_CNT        4
        #define NRF_MSK_OBSERVE_TX__PLOS_CNT        0xF0
        #define NRF_SFT_OBSERVE_TX__ARC_CNT         0
        #define NRF_MSK_OBSERVE_TX__ARC_CNT         0x0F
#define NRF_REG_CD              0x09  // 'Carrier Detect' register address
        #define NRF_SFT_CD__CD             (0)
        #define NRF_MSK_CD__CD             (1<<NRF_SFT_CD__CD)
#define NRF_REG_RX_ADDR_P0      0x0A  // 'RX address pipe0' register address
#define NRF_REG_RX_ADDR_P1      0x0B  // 'RX address pipe1' register address
#define NRF_REG_RX_ADDR_P2      0x0C  // 'RX address pipe2' register address
#define NRF_REG_RX_ADDR_P3      0x0D  // 'RX address pipe3' register address
#define NRF_REG_RX_ADDR_P4      0x0E  // 'RX address pipe4' register address
#define NRF_REG_RX_ADDR_P5      0x0F  // 'RX address pipe5' register address
#define NRF_REG_TX_ADDR         0x10  // 'TX address' register address
#define NRF_REG_RX_PW_P0        0x11  // 'RX payload width, pipe0' register address
#define NRF_REG_RX_PW_P1        0x12  // 'RX payload width, pipe1' register address
#define NRF_REG_RX_PW_P2        0x13  // 'RX payload width, pipe2' register address
#define NRF_REG_RX_PW_P3        0x14  // 'RX payload width, pipe3' register address
#define NRF_REG_RX_PW_P4        0x15  // 'RX payload width, pipe4' register address
#define NRF_REG_RX_PW_P5        0x16  // 'RX payload width, pipe5' register address
#define NRF_REG_FIFO_STATUS     0x17  // 'FIFO Status Register' register address
        #define NRF_SFT_FIFO_STATUS__RX_FIFO_EMPTY   (0)
        #define NRF_MSK_FIFO_STATUS__RX_FIFO_EMPTY   (1<<NRF_SFT_FIFO_STATUS__RX_FIFO_EMPTY)
        #define NRF_SFT_FIFO_STATUS__RX_FIFO_FULL    (1)
        #define NRF_MSK_FIFO_STATUS__RX_FIFO_FULL    (1<<NRF_SFT_FIFO_STATUS__RX_FIFO_FULL)
        #define NRF_SFT_FIFO_STATUS__TX_FIFO_EMPTY   (4)
        #define NRF_MSK_FIFO_STATUS__TX_FIFO_EMPTY   (1<<NRF_SFT_FIFO_STATUS__TX_FIFO_EMPTY)
        #define NRF_SFT_FIFO_STATUS__TX_FIFO_FULL    (5)
        #define NRF_MSK_FIFO_STATUS__TX_FIFO_FULL    (1<<NRF_SFT_FIFO_STATUS__TX_FIFO_FULL)

#define NRF_REG_DYNPD           0x1C  // Enable dynamic payload length
        #define NRF_SFT_DYNPD__DPL_P0          (0)
        #define NRF_MSK_DYNPD__DPL_P0          (1<<NRF_SFT_DYNPD__DPL_P0)
        #define NRF_SFT_DYNPD__DPL_P1          (1)
        #define NRF_MSK_DYNPD__DPL_P1          (1<<NRF_SFT_DYNPD__DPL_P1)
        #define NRF_SFT_DYNPD__DPL_P2          (2)
        #define NRF_MSK_DYNPD__DPL_P2          (1<<NRF_SFT_DYNPD__DPL_P2)
        #define NRF_SFT_DYNPD__DPL_P3          (3)
        #define NRF_MSK_DYNPD__DPL_P3          (1<<NRF_SFT_DYNPD__DPL_P3)
        #define NRF_SFT_DYNPD__DPL_P4          (4)
        #define NRF_MSK_DYNPD__DPL_P4          (1<<NRF_SFT_DYNPD__DPL_P4)
        #define NRF_SFT_DYNPD__DPL_P5          (5)
        #define NRF_MSK_DYNPD__DPL_P5          (1<<NRF_SFT_DYNPD__DPL_P5)
#define NRF_REG_FEATURE         0x1D  // Feature Register
        #define NRF_SFT_FEATURE__EN_DYN_ACK      (0)
        #define NRF_MSK_FEATURE__EN_DYN_ACK      (1<<NRF_SFT_FEATURE__EN_DYN_ACK)
        #define NRF_SFT_FEATURE__EN_ACK_PAY      (1)
        #define NRF_MSK_FEATURE__EN_ACK_PAY      (1<<NRF_SFT_FEATURE__EN_ACK_PAY)
        #define NRF_SFT_FEATURE__EN_DPL          (2)
        #define NRF_MSK_FEATURE__EN_DPL          (1<<NRF_SFT_FEATURE__EN_DPL)

//********************************************************************************************************************//
#define NRF_RX_FIFO_SIZE 32 
#define NRF_MAX_ADDR_LEN 5

#endif // NRF24L01_H
