/* Name: hwconfig.h
 * Project: NRF-LightCtrl
 * Author: SFR www.frydaysoft.de
 * Creation Date: 2013-05-01
 */

#ifndef __hwconfig_h_included__
#define __hwconfig_h_included__


/* ---------------------------- Hardware Config ---------------------------- */

/*
HW settings for tha Atmega8:
============================
NFR-CS-Pin      PB0   // see nrf_hw.h
NFR-CE-Pin      PB1   // see nrf_hw.h
Status_Led      PB2   // Led1Init()
MOSI            PB3   // SpiInit()
MISO            PB4   // SpiInit()
SCK             PB5   // SpiInit()

Bootloader-Pin  PC0   // HwLowInit(): future use
Select-Pin      PC1   // SelectInit()
NFR-IRQ-Pin     PC2   // see nrf_hw.h
LedTest-Pin     PC3   // LedTestInit()
SDA             PC4   // future use
SCL             PC5   // future use

LED-Port        PD    // InitOutLeds()

fuse: 0xC1, 0xA4
- Broun-out @ 2,7V
- Boot Flash section size=1024 words
- CKOPT fuse
- EESAVE=0
- int. RC Ocs. 8MHz, 6CK+64ms
*/

// *********************************************************************************
#define Led1Init()       PORTB |= (1 << PB2); DDRB |= (1 << PB2)
#define Led1Tog()        PORTB ^= (1 << PB2)
#define Led1On()         PORTB &= ~(1 << PB2)
#define Led1Off()        PORTB |= (1 << PB2)

#define SelectInit()     PORTC |= (1 << PC1); DDRC &= ~(1 << PC1)
#define IsSelectLow()    (PINC & (1 << PC1))
#define LedTestInit()    PORTC |= (1 << PC3); DDRC &= ~(1 << PC3)
#define IsLedTest()      !(PINC & (1 << PC3))

#define InitOutLeds()    PORTD |= 0xFF; DDRD |= 0xFF

//#define SetOutLeds(val)  PORTD = ~val
#define SetOutLeds(val)  PORTD = val

 

/* Example configuration: Port C bit 0 is connected to a jumper which ties
 * this pin to GND if the boot loader is requested. Initialization allows
 * several clock cycles for the input voltage to stabilize before
 * AppCondition() samples the value.
 * We use a function for HwLowInit() for convenience and a macro for
 * AppCondition() for efficiency.
 */

#ifndef __ASSEMBLER__   /* assembler cannot parse function definitions */
#include <util/delay.h>

  // ****** PC2
  static inline void  HwLowInit(void)
  {
      PORTC =  (1 << PC0); /* activate pull-up for key */
      DDRC &= ~(1 << PC0);
      _delay_us(10);       /* wait for levels to stabilize */
  }

  #define AppCondition()   (PINC & (1 << PC0))   // True if jumper is not set  --> low

#endif

/* ------------------------------------------------------------------------- */

#endif /* __hwconfig_h_included__ */
