/* Name: hwconfig.h
 * Project: NRF-LightCtrl
 * Author: SFR www.frydaysoft.de
 * Creation Date: 2013-05-01
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/boot.h>
#include <string.h>
#include <util/delay.h>

static void startBootloader() __attribute__((__noreturn__));

#include "nrf/nrf_types.h"
#include "nrf/nrf_drv.h"

#include "hwconfig.h"

//#define TX_TEST

/* ------------------------------------------------------------------------ */

#ifndef ulong
#   define ulong    unsigned long
#endif
#ifndef uint
#   define uint     unsigned int
#endif

// *********************************************************************************
static BYTE  NrfRxBuf[NRF_RX_FIFO_SIZE];

//-------------------------------------------------------
BYTE Nrf_RxSlaveAddr[] = {'T', 'x', '_', '0', '0'}; // nrf address tx master data
BYTE Nrf_TxSlaveAddr[] = {'R', 'x', '_', '0', '0'}; // nrf address rx master data

/* compatibility with ATMega88 and other new devices: */
#ifndef TCCR0
#define TCCR0   TCCR0B
#endif
#ifndef GICR
#define GICR    MCUCR
#endif

// ************************************************************************
// ***** startBootloader() ************************************************
// ************************************************************************
static void startBootloader()
{
    cli();
    wdt_enable(WDTO_1S);
    while(1) {;}
}


#define USE_T1
// ************************************************************************
// ***** TimerInit() ******************************************************
// ************************************************************************
void TimerInit(void)
{ 
#ifdef USE_T1
  // ***** stop Timer1
  TCCR1A = 0x00;
  TCCR1B = 0x00;   
  // ***** set Output Compare Register
  OCR1A  = 12500;  // 800000 / 64 * 0,1 = 18750
  TIFR   = (1<<OCF1A); 
  TCCR1B = 0x0B;   // clear Timer1 on CTC, prescaler 64
#else
#endif
}

// ************************************************************************
// ***** TimerPoll() ******************************************************
// ************************************************************************
BYTE TimerPoll()
{
#ifdef USE_T1
  if (TIFR & (1<<OCF1A))
  {
    TIFR  |= (1<<OCF1A); 
	  return 1;
  }
  return 0;
#else
  return 1;
#endif
}

// ************************************************************************
// ***** SpiInit() ********************************************************
// ************************************************************************
void SpiInit(void)
{
  volatile char IOReg;
  // Note: 
  // PB2 (/SS) will be used as output (CE of NRF chip) to avoid SPI slave select problem
  // ***** enable SPI in Master Mode with SCK = MCK/2 (4MHz)
  SPCR    = (1<<SPE)|(1<<MSTR);
  SPSR    = (1 << SPI2X);
  // ***** set PB3(MOSI), PB5(SCK) as output 
  DDRB    = (1<<PB3)|(1<<PB5);
  PORTB  |= (1<<PB4);
  DDRB   &= ~(1<<PB4);
  // ***** clear SPIF bit in SPSR
  IOReg   = SPSR;                 	
  IOReg   = SPDR;
}


// ************************************************************************
// ***** main() ***********************************************************
// ************************************************************************
int __attribute__((noreturn)) main(void)
{
    BYTE   steps, leds, status;

    wdt_enable(WDTO_1S); 
    cli();
    // ***** initialize hardware
    HwLowInit();
    SpiInit();          // call at first SpiInit() and then configure /SS as output
    Led1Init();
    NRF24L_SET_CE_LOW();
    NRF24L_SET_CS_HIGH();
    NRF24L_INIT_PINS();
    SelectInit();
    LedTestInit();
    InitOutLeds();
    TimerInit();
    steps = 2;
    while(steps)
    { // ***** 10ms nrf24l boot time
      wdt_reset();
      if (TimerPoll())
      {
        steps--;
      }
    }
    NrfHwInit(NRF_RF_PWR_0dBm, NRF_2MBPS, NRF_LNA_HCURR);
    // sei(); running in polling mode !!!

#ifdef TX_TEST
    NrfDrvSetTxMaster(2, Nrf_RxSlaveAddr, NRF_AW_5BYTES); 
    do
    { 
      wdt_reset();
      status = 0;
      if (TimerPoll())
      {
        leds = leds << 1;
        if (!leds)
        {
          leds = 0x1;
        }
        SetOutLeds(leds);
        NrfRxBuf[2] = leds;
        NrfDrvClearInt(NRF_ALL_IRQ);
        NrfDrvWriteFifoData(NrfRxBuf, 4);
        NrfHwTransmitPulse();
      }
    }while(AppCondition());
#else
    // ***** NRF on Channel 2 
    NrfDrvSetRxSlave(2, Nrf_RxSlaveAddr, NRF_AW_5BYTES); 
    steps = leds = 0; 
    // ***** main event loop 
    do
    { 
      wdt_reset();
      if (TimerPoll())
      {
        if (++steps >= 10)
        {
          Led1Tog();
          steps = 0;
          if (IsLedTest())
          {
            leds = leds << 1;
            if (!leds)
            {
              leds = 0x1;
            }
            SetOutLeds(leds);
          }
        }
      }
      if (NRF24L_IS_IRQ())
      {
        status = NrfHwStatus();
        if (status & NRF_RX_DR_IRQ)
        {
          NrfDrvClearInt(NRF_RX_DR_IRQ);
          status = NrfDrvGetRxSize();
          if (status)
          {
            Led1Tog();
            if (status > NRF_RX_FIFO_SIZE)
            {
              NrfHwFlushRx();
            }
            else
            { // ***** expect 0x03, 'L', xx, yy
              NrfDrvReadFifoData(&NrfRxBuf[0], status);
              if ((NrfRxBuf[0] == 0x03) && (NrfRxBuf[1] == 'L'))
              {
                if (IsSelectLow())
                {
                  SetOutLeds(NrfRxBuf[2]);
                }
                else
                {
                  SetOutLeds(NrfRxBuf[3]);
                }
              }
            }
          }
        }
      }
    }while(AppCondition());
#endif

    startBootloader();
}

