/* Name: nRF24L01.c
 * Project     : WinAmp USB LightCtrl via USB
 * Author      : frydaysoft.de (based on an example from Stanimir Bonev.)
 * File name   : nrf_drv.h
 * Description : nRF24L01 driver include file.
 *  
 * Copyright: (c) 2013 frydaysoft.de 
 * License  : Proprietary, free under certain conditions. See Documentation.
 *
 * Creation Date: 2013-04-27
 */

#ifndef NRF_DRV_H
#define NRF_DRV_H

#include "nrf24l01.h"
#include "nrf_hw.h"

//********************************************************************************************************************//
#define NRF_PIPE_0M (BYTE)(1<<0)
#define NRF_PIPE_1M (BYTE)(1<<1)
#define NRF_PIPE_2M (BYTE)(1<<2)
#define NRF_PIPE_3M (BYTE)(1<<3)
#define NRF_PIPE_4M (BYTE)(1<<4)
#define NRF_PIPE_5M (BYTE)(1<<5)
#define NRF_PIPE_1_TO_5_M (BYTE)(NRF_PIPE_1M | NRF_PIPE_2M | NRF_PIPE_3M | NRF_PIPE_4M | NRF_PIPE_5M )
#define NRF_ALL_PIPES_M (BYTE)(NRF_PIPE_0M | NRF_PIPE_1M | NRF_PIPE_2M | NRF_PIPE_3M | NRF_PIPE_4M | NRF_PIPE_5M )


//********************************************************************************************************************//
typedef enum {NRF_TX_MODE = 0, NRF_RX_MODE = !NRF_TX_MODE} NrfModeType ;

typedef enum {NRF_POWER_DOWN = 0, NRF_POWER_UP = !NRF_POWER_DOWN} NrfPowerType;

typedef enum {NRF_CRC8 = 0, NRF_CRC16 = !NRF_CRC8} NrfCrcType;

typedef enum {NRF_CRC_DISABLE = 0, NRF_CRC_ENABLE = !NRF_CRC_DISABLE} NrfCrcEnType;

typedef enum {NRF_AW_3BYTES = 1, NRF_AW_4BYTES, NRF_AW_5BYTES} NrfAddrWidthType;

typedef enum {NRF_ARD_250us  = (0<<4),  NRF_ARD_500us  = (1<<4),  NRF_ARD_750us  = (2<<4),
              NRF_ARD_1000us = (3<<4),  NRF_ARD_1250us = (4<<4),  NRF_ARD_1500us = (5<<4),
              NRF_ARD_1750us = (6<<4),  NRF_ARD_2000us = (7<<4),  NRF_ARD_2250us = (8<<4),
              NRF_ARD_2500us = (9<<4),  NRF_ARD_2750us = (10<<4), NRF_ARD_3000us = (11<<4),
              NRF_ARD_3250us = (12<<4), NRF_ARD_3500us = (13<<4), NRF_ARD_3750us = (14<<4),
              NRF_ARD_4000us = (15<<4)} NrfAutoDelayType;

typedef enum {NRF_DPL_DISABLE = 0, NRF_DPL_ENABLE = !NRF_DPL_DISABLE} NrfDynPayType;

typedef enum {NRF_DYN_ACK_DISABLE = 0, NRF_DYN_ACK_ENABLE = !NRF_DYN_ACK_DISABLE} NrfDynAckType;

typedef enum {NRF_ACK_PAY_DISABLE = 0, NRF_ACK_PAY_ENABLE = !NRF_ACK_PAY_DISABLE} NrfAckPayType;

typedef enum {NRF_ST_PWR_DOWN = 0 , NRF_ST_STAND_I, NRF_ST_STAND_II, NRF_ST_RX, NRF_ST_TX} NrfStateType;

//********************************************************************************************************************//
typedef enum {NRF_PIPE0 = 0, NRF_PIPE1, NRF_PIPE2, NRF_PIPE3, NRF_PIPE4, NRF_PIPE5} NrfPipeType;

//********************************************************************************************************************//
void NrfDrvMode(NrfModeType eMode);
void NrfDrvPower(NrfPowerType ePower);
void NrfDrvCrcType(NrfCrcType eCrc);
void NrfDrvCrcEn( NrfCrcEnType eEnable);
void NrfDrvMaskInt(BYTE Interrupt);
void NrfDrvUnmaskInt(BYTE Interrupt);
void NrfDrvEnAutoAck(BYTE Pipe);
void NrfDrvDisAutoAck(BYTE Pipe);
void NrfDrvEnRxAddr(BYTE Pipe);
void NrfDrvDisRxAddr(BYTE Pipe);
void NrfDrvAddrWidth(NrfAddrWidthType eAddrWidth);
void NrfDrvAutoRetDelay(NrfAutoDelayType eDelay);
void NrfDrvAutoRetCount(BYTE Count);
#define NrfDrvSetChannel(Channel)       NrfHwWriteRegByte(NRF_REG_RF_CH, (Channel & NRF_MSK_RF_CH__RANGE))
#define NrfDrvGetChannel()              NrfHwReadRegByte(NRF_REG_RF_CH)
void NrfDrvLNA(NrfLnaType eCurrent);
void NrfDrvOutPower(NrfOutPowerType ePower);
void NrfDrvDataRate(NrfDataRateType eRate);
#define NrfDrvClearInt(Interrupt)  NrfHwWriteRegByte(NRF_REG_STATUS, (Interrupt & (NRF_ALL_IRQ)))
BOOL NrfDrvSetRxAddr(NrfPipeType ePipe, BYTE *pAddr, BYTE Size);
BOOL NrfDrvSetTxAddr(BYTE *pAddr, BYTE Size);
BOOL NrfDrvRxPayWidth(NrfPipeType ePipe, BYTE PayWidth);
void NrfDrvEnDynPay(BYTE Pipe);
void NrfDrvDisDynPay(BYTE Pipe);
void NrfDrvDynPay(NrfDynPayType eDynPay);
void NrfDrvDynAck(NrfDynAckType eDynAck);
void NrfDrvAckPay(NrfAckPayType eAckPay);
#define NrfDrvGetRxSize()                 NrfHwReadRegByte(NRF_CMD_RD_RX_PLOAD_WID)
BYTE NrfDrvGetPipeAvb(void);
NrfStateType NrfDrvGetState(void);
#define NrfDrvGetFifoStatus()             NrfHwReadRegByte(NRF_REG_FIFO_STATUS)
BYTE NrfDrvGetIntID(void);
#define NrfDrvReadFifoData(pData, Size)   NrfHwReadReg(NRF_CMD_RD_RX_PLOAD, pData, Size)
#define NrfDrvReadFifoByte()              NrfHwReadRegByte(NRF_CMD_RD_RX_PLOAD)
#define NrfDrvWriteFifoData(pData, Size)  NrfHwWriteReg(NRF_CMD_WR_TX_PLOAD, pData, Size)
#define NrfDrvWriteFifoByte(Data)         NrfHwWriteRegByte(NRF_CMD_WR_TX_PLOAD, Data)

void NrfDrvSetPowerDown(void);
void NrfDrvSetRxSlave(BYTE channel, BYTE *pAddr, NrfAddrWidthType eAddrWidth);
void NrfDrvSetTxMaster(BYTE channel, BYTE *pAddr, NrfAddrWidthType eAddrWidth);

#endif // NRF_DRV_H
