/* 
 * Project     : WinAmp USB LightCtrl via USB
 * Author      : frydaysoft.de (based on an example from Stanimir Bonev.)
 * File name   : nrf_hw.h
 * Description : nRF24L01 low level driver include file
 *  
 * Copyright: (c) 2013 frydaysoft.de 
 * License  : Proprietary, free under certain conditions. See Documentation.
 *
 * Creation Date: 2013-04-27
 */

#ifndef NRF_HW_H
#define NRF_HW_H

//********************************************************************************************************************//
#define SAM7_TARGET       1
#define AVR_TARGET        2
#define ATMEGA8_RECEIVER  3


//********************************************************************************************************************//
#define NRF_TARGET   ATMEGA8_RECEIVER


//********************************************************************************************************************//
#if SAM7_TARGET == NRF_TARGET
  #include <AT91SAM7S64.h>
  
  // evaluation board olimex SAM7-P64
  // The NRF24 is connected on the UEXT connector

  // ***** HW specific
  #define NRF24L_IRQ_MASK     (AT91C_PIO_PA3)
  #define NRF24L_CE_MASK      (AT91C_PIO_PA4)
  #define NRF24L_CS_MASK      (AT91C_PIO_PA31)
  
  #define NRF24L_INIT_PINS()    AT91C_BASE_PIOA->PIO_PER  = NRF24L_CS_MASK | NRF24L_CE_MASK; AT91C_BASE_PIOA->PIO_OER  = NRF24L_CS_MASK | NRF24L_CE_MASK

  #define NRF24L_GET_CE()        ((AT91C_BASE_PIOA->PIO_PDSR & NRF24L_CE_MASK) != 0)
  #define NRF24L_SET_CE_LOW()    (AT91C_BASE_PIOA->PIO_CODR = NRF24L_CE_MASK)
  #define NRF24L_SET_CE_HIGH()	 (AT91C_BASE_PIOA->PIO_SODR = NRF24L_CE_MASK)
  
  #define NRF24L_SET_CS_LOW()    (AT91C_BASE_PIOA->PIO_CODR = NRF24L_CS_MASK)
  #define NRF24L_SET_CS_HIGH()   (AT91C_BASE_PIOA->PIO_SODR = NRF24L_CS_MASK)

  #define NRF24L_IS_IRQ()        (!(AT91C_BASE_PIOA->PIO_PDSR & NRF24L_IRQ_MASK))

  #define NRF_TX_PULSE_DLY       80  // >10us

#elif AVR_TARGET == NRF_TARGET

  // AVR board USBASP V2.0 (Lcsoft Studio) with Atmega8
  // The NRF24 is connected on the programming connector
  #include <avr/io.h> 

  // ***** HW specific
  #define NRF24L_IRQ_MASK     (1 << PD1)
  #define NRF24L_CE_MASK      (1 << PB2)
  #define NRF24L_CS_MASK      (1 << PD0)

  #define NRF24L_INIT_PINS()  PORTD |= (NRF24L_IRQ_MASK); DDRD &= ~(NRF24L_IRQ_MASK); PORTD |= (NRF24L_CS_MASK); PORTB |= (NRF24L_CE_MASK); DDRD |= (NRF24L_CS_MASK); DDRB |= (NRF24L_CE_MASK)

  #define NRF24L_GET_CE()        ((PINB & NRF24L_CE_MASK) != 0)
  #define NRF24L_SET_CE_LOW()    (PORTB &= ~NRF24L_CE_MASK)
  #define NRF24L_SET_CE_HIGH()	 (PORTB |= NRF24L_CE_MASK)
  
  #define NRF24L_SET_CS_LOW()    (PORTD &= ~NRF24L_CS_MASK)
  #define NRF24L_SET_CS_HIGH()   (PORTD |= NRF24L_CS_MASK)

  #define NRF24L_IS_IRQ()        (!(PIND & NRF24L_IRQ_MASK))

  #define NRF_TX_PULSE_DLY       20  // >10us: 20=15us, 40=30us @12MHz

#elif ATMEGA8_RECEIVER == NRF_TARGET

  // simple HW with Atmega8
  #include <avr/io.h> 
  /*
  HW settings for tha Atmega8:
  ============================
  NFR-CS-Pin      PB0   // see nrf_hw.h
  NFR-CE-Pin      PB1   // see nrf_hw.h
  Status_Led      PB2   // Led1Init()
  MOSI            PB3   // SpiInit()
  MISO            PB4   // SpiInit()
  SCK             PB5   // SpiInit()

  Bootloader-Pin  PC0   // HwLowInit(): future use
  Select-Pin      PC1   // SelectInit()
  NFR-IRQ-Pin     PC2   // see nrf_hw.h
  LedTest-Pin     PC3   // LedTestInit()
  SDA             PC4   // future use
  SCL             PC5   // future use

  LED-Port        PD    // InitOutLeds()
  */

  // ***** HW specific
  #define NRF24L_IRQ_MASK     (1 << PC2)
  #define NRF24L_CE_MASK      (1 << PB1)
  #define NRF24L_CS_MASK      (1 << PB0)

  #define NRF24L_INIT_PINS()  PORTC |= (NRF24L_IRQ_MASK); DDRC &= ~(NRF24L_IRQ_MASK); PORTB |= (NRF24L_CS_MASK | NRF24L_CE_MASK); DDRB |= (NRF24L_CS_MASK | NRF24L_CE_MASK)

  #define NRF24L_GET_CE()        ((PINB & NRF24L_CE_MASK) != 0)
  #define NRF24L_SET_CE_LOW()    (PORTB &= ~NRF24L_CE_MASK)
  #define NRF24L_SET_CE_HIGH()	 (PORTB |= NRF24L_CE_MASK)
  
  #define NRF24L_SET_CS_LOW()    (PORTB &= ~NRF24L_CS_MASK)
  #define NRF24L_SET_CS_HIGH()   (PORTB |= NRF24L_CS_MASK)

  #define NRF24L_IS_IRQ()        (!(PINC & NRF24L_IRQ_MASK))

  #define NRF_TX_PULSE_DLY       20  // >10us: 30=15us, 60=30us @8MHz

#else
  #error "unknown target"
#endif

//********************************************************************************************************************//
typedef enum 
{
  NRF_RF_PWR_18dBm = (0<<1), NRF_RF_PWR_12dBm = (1<<1), NRF_RF_PWR_6dBm  = (2<<1), NRF_RF_PWR_0dBm  = (3<<1)
} NrfOutPowerType;

typedef enum 
{
  NRF_1MBPS = 0, NRF_2MBPS
} NrfDataRateType;

typedef enum 
{
  NRF_LNA_LCURR = 0, NRF_LNA_HCURR = !NRF_LNA_LCURR
} NrfLnaType;

//********************************************************************************************************************//
void NrfHwInit(NrfOutPowerType ePower, NrfDataRateType eRate, NrfLnaType eCurrent);
BYTE NrfHwLastResult(void);
BYTE NrfHwStatus(void);
void NrfHwWriteReg(BYTE Addr, BYTE *pData, BYTE Size);
void NrfHwWriteRegByte(BYTE Addr, BYTE Data);
void NrfHwReadReg(BYTE Addr, BYTE *pData, BYTE Size);
BYTE NrfHwReadRegByte(BYTE Addr);
void NrfHwWriteActRegByte(BYTE Addr, BYTE Data);
BYTE NrfHwReadActRegByte(BYTE Addr);
void NrfHwFlushRx(void);
void NrfHwFlushTx(void);

//********************************************************************************************************************//
// HW specific: I/O lines 5V tolerant, VCC max 3.6V, Max 8Mbps SPI
void NrfHwTransmitPulse(void);
BYTE NrfHwSpiSend(BYTE data);

#endif // NRF_HW_H
